__title__ = "sfcc connector"
__description__ = "Python connector for Salesforce Commerce Cloud via Open Commerce API"
__url__ = "https://github.com/edro15/sfcc-connector"
__version__ = "1.0.2"
__author__ = "Erica Pescio"
__author_email__ = "edro15@users.noreply.github.com"
__license__ = "MIT"
__maintainer__ = "Erica Pescio"
__maintainer_email__ = "edro15@users.noreply.github.com"
__keywords__ = ["python", "salesforce commerce cloud", "ocapi", "open commerce api", "sfcc"]
